@@ -18,7 +18,6 @@
 
 package org.apache.roller.weblogger.ui.rendering.model;
 
-import java.io.IOException;
 import java.sql.Timestamp;
 import java.util.Collections;
 import java.util.Date;
@@ -26,30 +25,19 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeMap;
 import java.util.TreeSet;
-
 import org.apache.commons.text.StringEscapeUtils;
-import org.apache.commons.lang3.StringUtils;
-import org.apache.lucene.document.Document;
-import org.apache.lucene.search.ScoreDoc;
-import org.apache.lucene.search.TopFieldDocs;
 import org.apache.roller.util.DateUtil;
 import org.apache.roller.weblogger.WebloggerException;
 import org.apache.roller.weblogger.business.URLStrategy;
-import org.apache.roller.weblogger.business.WeblogEntryManager;
-import org.apache.roller.weblogger.business.Weblogger;
 import org.apache.roller.weblogger.business.WebloggerFactory;
-import org.apache.roller.weblogger.business.search.FieldConstants;
 import org.apache.roller.weblogger.business.search.IndexManager;
-import org.apache.roller.weblogger.business.search.operations.SearchOperation;
-import org.apache.roller.weblogger.config.WebloggerRuntimeConfig;
-import org.apache.roller.weblogger.pojos.WeblogEntry;
+import org.apache.roller.weblogger.business.search.SearchResultList;
 import org.apache.roller.weblogger.pojos.WeblogEntryWrapperComparator;
 import org.apache.roller.weblogger.pojos.wrapper.WeblogCategoryWrapper;
 import org.apache.roller.weblogger.pojos.wrapper.WeblogEntryWrapper;
 import org.apache.roller.weblogger.ui.rendering.pagers.SearchResultsPager;
 import org.apache.roller.weblogger.ui.rendering.pagers.WeblogEntriesPager;
 import org.apache.roller.weblogger.ui.rendering.util.WeblogSearchRequest;
-import org.apache.roller.weblogger.util.I18nMessages;
 
 /**
  * Extends normal page renderer model to represent search results.
@@ -65,7 +53,7 @@ public class SearchResultsModel extends PageModel {
 	private URLStrategy urlStrategy = null;
 
 	// the actual search results mapped by Day -> Set of entries
-    private final Map<Date, Set<WeblogEntryWrapper>> results = new TreeMap<>(Collections.reverseOrder());
+    private Map<Date, Set<WeblogEntryWrapper>> results = new TreeMap<>(Collections.reverseOrder());
 
 	// the pager used by the 3.0+ rendering system
 	private SearchResultsPager pager = null;
@@ -74,8 +62,7 @@ public class SearchResultsModel extends PageModel {
 	private int offset = 0;
 	private int limit = 0;
 	private Set<String> categories = new TreeSet<String>();
-	private boolean websiteSpecificSearch = true;
-	private String errorMessage = null;
+	private String errorMessage = "";
 
 	@Override
 	public void init(Map<String, Object> initData) throws WebloggerException {
@@ -83,8 +70,7 @@ public class SearchResultsModel extends PageModel {
 		// we expect the init data to contain a searchRequest object
 		searchRequest = (WeblogSearchRequest) initData.get("searchRequest");
 		if (searchRequest == null) {
-			throw new WebloggerException(
-					"expected searchRequest from init data");
+			throw new WebloggerException("expected searchRequest from init data");
 		}
 
 		// look for url strategy
@@ -98,55 +84,59 @@ public class SearchResultsModel extends PageModel {
 
 		// if there is no query, then we are done
 		if (searchRequest.getQuery() == null) {
-			pager = new SearchResultsPager(urlStrategy, searchRequest, results,
-					false);
+			pager = new SearchResultsPager(urlStrategy, searchRequest, results, false);
 			return;
 		}
 
 		// setup the search
-		IndexManager indexMgr = WebloggerFactory.getWeblogger()
-				.getIndexManager();
+		IndexManager indexMgr = WebloggerFactory.getWeblogger().getIndexManager();
+		try {
+			SearchResultList searchResultList = indexMgr.search(
+				searchRequest.getQuery(),
+				searchRequest.getWeblogHandle(),
+				searchRequest.getWeblogCategoryName(),
+				searchRequest.getLocale(),
+				searchRequest.getPageNum(),
+				RESULTS_PER_PAGE,
+				urlStrategy
+			);
+			hits = searchResultList.getResults().size();
+			offset = searchResultList.getOffset();
+			limit = searchResultList.getLimit();
+			categories = searchResultList.getCategories();
 
-		SearchOperation search = new SearchOperation(indexMgr);
-		search.setTerm(searchRequest.getQuery());
-
-		if (WebloggerRuntimeConfig.isSiteWideWeblog(searchRequest
-				.getWeblogHandle())) {
-			this.websiteSpecificSearch = false;
-		} else {
-			search.setWebsiteHandle(searchRequest.getWeblogHandle());
-		}
-
-		if (StringUtils.isNotEmpty(searchRequest.getWeblogCategoryName())) {
-			search.setCategory(searchRequest.getWeblogCategoryName());
-		}
-
-		if (searchRequest.getLocale() != null) {
-			search.setLocale(searchRequest.getLocale());
-		}
-
-		// execute search
-		indexMgr.executeIndexOperationNow(search);
-
-		if (search.getResultsCount() == -1) {
-			// this means there has been a parsing (or IO) error
-			this.errorMessage = I18nMessages.getMessages(
-					searchRequest.getLocaleInstance()).getString(
-					"error.searchProblem");
-		} else {
-
-			TopFieldDocs docs = search.getResults();
-			ScoreDoc[] hitsArr = docs.scoreDocs;
-			this.hits = search.getResultsCount();
-
-			// Convert the Hits into WeblogEntryData instances.
-			convertHitsToEntries(hitsArr, search);
+			Timestamp now = new Timestamp(new Date().getTime());
+			for (WeblogEntryWrapper entry : searchResultList.getResults()) {
+				if (entry.getPubTime().before(now)) {
+					addEntryToResults(results, entry);
+				}
+			}
 
+		} catch (WebloggerException we) {
+			errorMessage = we.getMessage();
 		}
 
 		// search completed, setup pager based on results
-		pager = new SearchResultsPager(urlStrategy, searchRequest, results,
-				(hits > (offset + limit)));
+		pager = new SearchResultsPager(
+			urlStrategy, searchRequest, results, (hits > (offset + limit)));
+	}
+
+	private void addEntryToResults(
+		Map<Date, Set<WeblogEntryWrapper>> results,
+		WeblogEntryWrapper entry) {
+
+		// convert entry's each date to midnight (00m 00h 00s)
+		Date midnight = DateUtil.getStartOfDay(entry.getPubTime());
+
+		// ensure we do not get duplicates from Lucene by
+		// using a Set Collection. Entries sorted by pubTime.
+		Set<WeblogEntryWrapper> set = results.get(midnight);
+		if (set == null) {
+			// date is not mapped yet, so we need a new Set
+			set = new TreeSet<>(new WeblogEntryWrapperComparator());
+			results.put(midnight, set);
+		}
+		set.add(entry);
 	}
 
 	/**
@@ -169,86 +159,6 @@ public class SearchResultsModel extends PageModel {
 		return pager;
 	}
 
-	/**
-	 * Convert hits to entries.
-	 * 
-	 * @param hits
-	 *            the hits
-	 * @param search
-	 *            the search
-	 * @throws WebloggerException
-	 *             the weblogger exception
-	 */
-	private void convertHitsToEntries(ScoreDoc[] hits, SearchOperation search)
-			throws WebloggerException {
-
-		// determine offset
-		this.offset = searchRequest.getPageNum() * RESULTS_PER_PAGE;
-		if (this.offset >= hits.length) {
-			this.offset = 0;
-		}
-
-		// determine limit
-		this.limit = RESULTS_PER_PAGE;
-		if (this.offset + this.limit > hits.length) {
-			this.limit = hits.length - this.offset;
-		}
-
-		try {
-			Set<String> categorySet = new TreeSet<>();
-			Weblogger roller = WebloggerFactory.getWeblogger();
-			WeblogEntryManager weblogMgr = roller.getWeblogEntryManager();
-
-			WeblogEntry entry;
-			Document doc;
-			String handle;
-			Timestamp now = new Timestamp(new Date().getTime());
-			for (int i = offset; i < offset + limit; i++) {
-				doc = search.getSearcher().doc(hits[i].doc);
-				handle = doc.getField(FieldConstants.WEBSITE_HANDLE)
-						.stringValue();
-
-                entry = weblogMgr.getWeblogEntry(doc.getField(
-                        FieldConstants.ID).stringValue());
-
-                if (!(websiteSpecificSearch && handle.equals(searchRequest.getWeblogHandle()))
-                        && doc.getField(FieldConstants.CATEGORY) != null) {
-                    categorySet.add(doc.getField(FieldConstants.CATEGORY).stringValue());
-                }
-
-				// maybe null if search result returned inactive user
-				// or entry's user is not the requested user.
-				// but don't return future posts
-				if (entry != null && entry.getPubTime().before(now)) {
-					addEntryToResults(WeblogEntryWrapper.wrap(entry,
-							urlStrategy));
-				}
-			}
-
-			if (!categorySet.isEmpty()) {
-				this.categories = categorySet;
-			}
-		} catch (IOException e) {
-			throw new WebloggerException(e);
-		}
-	}
-
-	private void addEntryToResults(WeblogEntryWrapper entry) {
-
-		// convert entry's each date to midnight (00m 00h 00s)
-		Date midnight = DateUtil.getStartOfDay(entry.getPubTime());
-
-		// ensure we do not get duplicates from Lucene by
-		// using a Set Collection. Entries sorted by pubTime.
-		Set<WeblogEntryWrapper> set = this.results.get(midnight);
-		if (set == null) {
-			// date is not mapped yet, so we need a new Set
-			set = new TreeSet<>(new WeblogEntryWrapperComparator());
-			this.results.put(midnight, set);
-		}
-		set.add(entry);
-	}
-
 	public String getTerm() {
 		String query = searchRequest.getQuery();
         return (query == null)
@@ -291,8 +201,7 @@ public class SearchResultsModel extends PageModel {
 	@Override
 	public WeblogCategoryWrapper getWeblogCategory() {
 		if (searchRequest.getWeblogCategory() != null) {
-			return WeblogCategoryWrapper.wrap(
-					searchRequest.getWeblogCategory(), urlStrategy);
+			return WeblogCategoryWrapper.wrap(searchRequest.getWeblogCategory(), urlStrategy);
 		}
 		return null;
 	}
